import * as React from 'react';
import * as ReactDOM from 'react-dom';
// We don't let knockout initialize the React elements, so we need to get the context from the parent element
function getKnockoutContext(element) {
    while (element) {
        const cx = ko.contextFor(element);
        if (cx)
            return cx;
        element = element.parentElement;
    }
    throw new Error("Could not find knockout context");
}
/** React wrapper for knockout `ko.renderTemplate` function.
 * Specify the `templateName` property to select which template should be rendered.
 * Optionally, you can use the `viewModel` or `getChildContext` property to set a data context for the template. */
export class KnockoutTemplateReactComponent extends React.Component {
    constructor() {
        super(...arguments);
        this.wrapRef = React.createRef();
        this.templateName = ko.observable();
    }
    // TODO: how to dispose the template?
    // componentWillUnmount() {
    // }
    componentDidMount() {
        this.initializeTemplate();
    }
    initializeTemplate() {
        const e = this.wrapRef.current;
        let context = getKnockoutContext(e);
        if (this.props.getChildContext) {
            context = this.props.getChildContext(context);
        }
        else if (this.props.viewModel !== undefined) {
            const updateEvent = new dotvvm.DotvvmEvent("templateInReact.newState");
            this.viewModelStateManager = new dotvvm.StateManager(this.props.viewModel, updateEvent);
            context = context.createChildContext(this.viewModelStateManager.stateObservable);
        }
        this.updateStuff();
        ko.renderTemplate(this.templateName, context, {}, e);
    }
    componentDidUpdate() {
        this.updateStuff();
    }
    updateStuff() {
        if (this.templateName() !== this.props.templateName)
            this.templateName(this.props.templateName);
        if (this.viewModelStateManager) {
            this.viewModelStateManager.setState(this.props.viewModel);
        }
    }
    render() {
        return React.createElement(this.props.wrapperTag, Object.assign({ ref: this.wrapRef }, this.props.wrapperAttributes));
    }
}
KnockoutTemplateReactComponent.defaultProps = {
    wrapperTag: "div"
};
/** Converts React component to DotVVM component usable through `<js:MyComponent />` syntax (or the JsComponent class)
 * See [the complete guide](https://www.dotvvm.com/docs/4.0/pages/concepts/client-side-development/integrate-third-party-controls/react).
 *
 * The component will receive all properties, commands and templates as it's React props.
 *  * Properties are plain JS objects and values, notably they don't contain any knockout observables
 *  * Commands are functions returning a promise, optionally expecting arguments if they were specified in the dothtml markup
 *  * Templates are only string IDs which can be passed to the `<KnockoutTemplateReactComponent templateName={props.theTemplate} />` component
 *
 * Additional property `setProps` is passed to the component, which can be used to update the component's properties (if the bound expression is updatable, otherwise it will throw an error).
 * * Usage: `props.setProps({ myProperty: props.myProperty + 1 })`
 */
export const registerReactControl = (ReactControl, defaultProps = {}) => ({
    create: (elm, props, commands, templates, setPropsRaw) => {
        const initialProps = Object.assign(Object.assign(Object.assign({ setProps }, defaultProps), commands), templates);
        let currentProps = Object.assign(Object.assign({}, initialProps), props);
        rerender();
        return {
            updateProps(updatedProps) {
                currentProps = Object.assign(Object.assign({}, currentProps), updatedProps);
                rerender();
            },
            dispose() {
                ReactDOM.unmountComponentAtNode(elm);
            }
        };
        function rerender() {
            ReactDOM.render(React.createElement(ReactControl, Object.assign({}, currentProps)), elm);
        }
        function setProps(updatedProps) {
            currentProps = Object.assign(Object.assign({}, currentProps), updatedProps);
            setPropsRaw(updatedProps);
            rerender();
        }
    }
});
//# sourceMappingURL=dotvvm-react.js.map